/*
 * test_main_utils.cpp
 *
 *  Created on: May 25, 2016
 *      Author: vlegrand
 */

#include <iostream>
#include <cassert>
#include <string>
#include <cmath>


using namespace std;

#include "rock_commons.h"
#include "main_utils.h"
#include "ROCKparams.h"

void test_processIOFileArgs() {
    std::vector<IO_fq_files> single_files;
    vector<PE_files> v_PE_files;
    std::vector<string> v_input_lines;
    std::vector<string> v_output_lines;
    ROCKparams main_parms;
    int f_id=0;
    int ret=main_parms.loadInOutFileArgs("../test/data/unit/list_input1.txt","../test/data/unit/list_output1.txt",v_input_lines,v_output_lines);
    assert(ret==EXIT_SUCCESS);
    ret=main_parms.processInOutFileArgs(v_input_lines,v_output_lines,single_files,v_PE_files,f_id);
    assert(ret==EXIT_FAILURE);
    f_id=0;
    v_PE_files.clear();
    single_files.clear();
    v_input_lines.clear();
    v_output_lines.clear();
    ret=main_parms.loadInOutFileArgs("../test/data/unit/list_input2.txt","../test/data/unit/list_output2.txt",v_input_lines,v_output_lines);
    ret=main_parms.processInOutFileArgs(v_input_lines,v_output_lines,single_files,v_PE_files,f_id);
    assert(ret==EXIT_FAILURE);
    f_id=0;
    v_PE_files.clear();
    single_files.clear();
    v_input_lines.clear();
    v_output_lines.clear();
    ret=main_parms.loadInOutFileArgs("../test/data/unit/list_input3.txt","../test/data/unit/list_output3.txt",v_input_lines,v_output_lines);
    ret=main_parms.processInOutFileArgs(v_input_lines,v_output_lines,single_files,v_PE_files,f_id);
    assert(ret==EXIT_SUCCESS);
    assert(f_id==14);
    assert(single_files.size()==2);
    assert(v_PE_files.size()==6);
    IO_fq_files s=single_files[0];
    assert(s.in_fq_file.compare("fifi")==0);
    assert(s.out_fq_file.compare("ofifi")==0);
    PE_files s2=v_PE_files[5];
    assert(s2.PE2.in_fq_file.compare("nono")==0);
    assert(s2.PE2.out_fq_file.compare("onono")==0);

    v_output_lines.clear();
    v_input_lines.clear();
    v_PE_files.clear();
    single_files.clear();
    ret=main_parms.loadInOutFileArgs("../test/data/unit/list_input3.txt","",v_input_lines,v_output_lines);
    ret=main_parms.processInOutFileArgs(v_input_lines,v_output_lines,single_files,v_PE_files,f_id);
    s=single_files[0];
    assert(s.out_fq_file.compare("fifi.rock.fq")==0);
    s2=v_PE_files[5];
    assert(s2.PE2.out_fq_file.compare("nono.rock.fq")==0);

}


void test_getBestLambdaForN() {
    unsigned long nb_k_mer=5000000000;
    int best=getBestLambdaForN(nb_k_mer,2,UINT_MAX,0.05);
    assert(best==2);
    nb_k_mer=200000000;
    best=getBestLambdaForN(nb_k_mer,5,UINT_MAX,0.1);
    assert(best==1);
    nb_k_mer=600000000;
    best=getBestLambdaForN(nb_k_mer,70,UINT_MAX,0.1);
    assert(best==1);
    nb_k_mer=2000000000;
    best=getBestLambdaForN(nb_k_mer,10,UINT_MAX,0.1);
    assert(best==1);
    nb_k_mer=10000000000;
    best=getBestLambdaForN(nb_k_mer,5,UINT_MAX,0.1);
    //cout<<best<<endl;
    assert(best==1);
    best=getBestLambdaForN(nb_k_mer,1,UINT_MAX,0.05);
    //cout<<best<<endl;
    assert(best==8);

    nb_k_mer=50000000000;
    best=getBestLambdaForN(nb_k_mer,5,UINT_MAX,0.1);// this returns 90
    assert(best==90);
    best=getBestLambdaForN(nb_k_mer,50,UINT_MAX,0.1);
    assert(best==1);

    best=getBestLambdaForN(nb_k_mer,50,UINT_MAX,0.05);
    assert(best==1);
    best=getBestLambdaForN(nb_k_mer,10,UINT_MAX,0.05);
}

void test_getCollisionProba() {
	float p=getCollisionProba(2,5000000000,UINT_MAX,2);
	assert(round(p*10000)==127);
	p=getCollisionProba(2,5000000000,UINT_MAX,1);
	assert(p=0.1128);
    p =getCollisionProba(1,2,UINT_MAX,3);
    //cout<<p<<endl;
    assert(p==0.0);
    p =getCollisionProba(1,1000000000,UINT_MAX,1);
    assert(floor(p*1000+0.5)==23);
    p =getCollisionProba(5,1000000000,UINT_MAX,1);
    assert(floor(p*1000+0.5)==0);
    p =getCollisionProba(5,50000000000,UINT_MAX,1);
    assert(floor(p*1000+0.5)==975);
}

void test_gammln() {
    float a;
    a=gammln(1);
    assert(exp(a)==1);
    a=gammln(2);
    assert(exp(a)==1);
    a=gammln(3);
    assert(exp(a)==2);
    a=gammln(4);
    assert(exp(a)==6);
    a=gammln(5);
    assert(exp(a)==24);
    a=gammln(6);
    float tmp=exp(a);
    float tmp2=tmp*10000;
    float tmp3=round(tmp2)/10000;
    //printf("%s \n",tmp3);
    assert(tmp3==120);
   /* unsigned long n=5000;
    unsigned long m=n+1;
    a=gammln(m);
    float b=gammln(n);
    double truc=exp(a-b);
    assert(truc==n);
    */
    
}

void test_pmf() {
    unsigned long nb_k_mers=5000000000;
    float p=pmf(0,nb_k_mers,UINT_MAX);
    assert(round(p*10000)==3122);
    p=pmf(1,nb_k_mers,UINT_MAX);
    assert(round(p*10000)==3634);
    p=pmf(2,nb_k_mers,UINT_MAX);
    assert(round(p*10000)==2115);
}

void test_ccdf() {
	unsigned long nb_k_mers=5000000000;
	double res=ccdf(2,nb_k_mers,UINT_MAX);
	assert(round(res*10000)==1128);
}






int main() {
    cout<<"testing main_utils"<<endl;
    cout<<"testing processing of IOFile arguments ( following 2 error messages are what is expected; don't worry)."<<endl;
    test_processIOFileArgs();
    cout<<"testing the gammln function"<<endl;
    test_gammln();
    cout<<"testing the pmf function"<<endl;
    test_pmf();
    cout<<"testing the ccdf function"<<endl;
    test_ccdf();
    cout<<"testing computation of collision probability."<<endl;
    test_getCollisionProba();
    cout<<"test computation of the bast lambda value depending on nb distinct k-mers."<<endl;
    test_getBestLambdaForN();
    cout<<"done"<<endl;
}


